/*
 StrobeMsg.java
 Clement Jonquet - PhD student in computer science - LIRMM - July 2005
*/

package madkit.strobekit;

import madkit.kernel.Message;

/**
 * This class define the type of message exchanged by Strobe agents. A Strobe message is logically viewed as a set of four attributes (sender receiver performative content).
 * <p>
 * This class contains methods for: 
 * <ul>
 * 	<li> accessing Strobe message attibutes
 * 	<li> conversing Strobe message to <code>String</code>
 *  <li> 
 * </ul>
 * 
 * 
 * @author Clement Jonquet - Mail to <a href="mailto:jonquet@lirmm.fr">jonquet@lirmm.fr</a>
 * @version 0.1	01/30/2006
 */

public class StrobeMsg extends Message{

//	-----------	
//	Attributes
//	-----------
	
	// SENDER, RECEIVER the 2 agents concerned by the message
	//AgentAddress sender, receiver;
	
	// PERFO, CONTENT, 2 strings
	String  perfo, content;

//	--------
//  Methods
//	--------	
	
    //	-------------	
    //  Constructors
    //	-------------	

	/** Construt a Strobe message with the performative <code>p</code> and the content <code>c</code>.*/
	public StrobeMsg(String p, String c){
		perfo = p;
		content = c;
	}
	
	//	----------	
	//  Accessors
	//	----------	
	
	/** Return the Strobe message performative.*/
	public String getPerfo(){return perfo;}
	
	/** Return the Strobe message content.*/
	public String getContent(){return content;}
	
	/** Return the Strobe message as a <code>String</code>.*/
	public String toString(){
		return "\t SENDER: "+getSender().toString()+"\n \t\t RECEIVER: "+getReceiver().toString()+"\n \t\t PERFORMATIVE: "+perfo+"\n \t\t CONTENT: "+content;
	}
	
	// Tjrs a determiner... est ce que un message Scheme aura comme champ pour identifi l'agent un nom ou une agentaddress, du au probleme avec le format de l'agentaddress (@) en Scheme
	
	/** Tansform the Strobe message in a Scheme expression such as <code>(strobemsg sender receiver performative content)</code>.*/
	public String toSchemeExp(){
		// With agent address
		//return "(StrobeMsg "+getSender().toString()+" "+getReceiver().toString()+" "+perfo+" "+content+")";
		// With name
		return "(strobemsg "+getSender().getName()+" "+getReceiver().getName()+" "+perfo+" "+content+")";
	}
}
