;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; strobemsg.scm - Clement JONQUET - PhD student in Computer Science - LIRMM - July 2005 ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; DESCRIPTION :
; This files defines the evaluate-strobemsg function to eval message.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(TEST-TAG TAG)-> function, which taste if the car of an expression is the same symbol as tag   
(define (test-tag tag)
  (lambda (exp) (if (pair? exp) (eq? (car exp) tag) #f)))

; (STROBEMSG? EXP)-> BOOL, true if the tag of an expression is "strobemsg"
(define (strobemsg? exp) 
  ((test-tag 'strobemsg) exp))
; a strobemsg is a list such as (strobemsg sender receiver perform content)

;;;;;;;;;;;;;;; STROBEMSG EXPRESSIONS EVALUATION ;;;;;;;;;;;;;;;

; (EVALUATE-STROBEMSG MSG)-> EXP, a scheme expression which could be the result of the evaluation of the message (a data) or a list of message (list of strobemsg) as answer of the evaluated message
(define (evaluate-strobemsg e)
 ;(print-strobemsg e)
  (let ((sender (sender-part e))
        (receiver (receiver-part e))
        (perform (perform-part e))
	(content (content-part e)))
    (cond ((eq? perform 'assertion)
;           (list 'strobemsg-list-answer (make-strobemsg receiver sender 'ack (eval content))))
           (make-strobemsg receiver sender 'ack (eval content)))
          ((eq? perform 'request)
;           (list 'strobemsg-list-answer (make-strobemsg receiver sender 'answer (eval content))))
           (make-strobemsg receiver sender 'answer (eval content)))
          ((eq? perform 'order)
;           (list 'strobemsg-list-answer (make-strobemsg receiver sender 'executed (eval content))))
           (make-strobemsg receiver sender 'executed (eval content)))
          ((eq? perform 'ack) (analyze-answer content))
          ((eq? perform 'answer) (analyze-answer content))
          ((eq? perform 'executed) (analyze-answer content))
          ;((eq? perform 'broadcast)
          ; (map-un (lambda (x) (make-strobemsg receiver x (first content) (second content)))
          ;         current-conversations))
          ((eq? perform 'no-such-performative) (performative-to-teach content sender receiver))
          (else (list (make-strobemsg receiver sender 'no-such-performative perform))))))

; (MAKE-STROBEMSG SENDER RECEIVER PERFORM CONTENT)-> STROBEMSG-EXP
(define (make-strobemsg sender receiver perform content) (list 'strobemsg sender receiver perform content))

; (SENDER-PART, RECEIVER-PART, PERFORM-PART, CONTENT-PART STROBEMSG-EXP)-> AGENT-ADD, AGENT-ADD, SYMBOL, EXP
(define (sender-part strobemsg-exp) (second strobemsg-exp))
(define (receiver-part strobemsg-exp) (third strobemsg-exp))
(define (perform-part strobemsg-exp) (fourth strobemsg-exp))
;(define (content-part strobemsg-exp) (fifth strobemsg-exp))
(define (content-part strobemsg-exp) (if (> (length strobemsg-exp) 4) (fifth strobemsg-exp) '*void*))

; (ANALYZE-ANSWER EXP)->
(define (analyze-answer content)
  content)

; (PERFORMATIVE-TO-TEACH PERFORM)-> LIST OF MSG
(define (performative-to-teach unknownperform sender receiver)
  ())

; (PRINT-STROBEMSG STROBEMSG-EXP)-> nothing
(define (print-strobemsg strobemsg)
  (display "STROBEMSG : ")(display (sender-part strobemsg))(display (receiver-part strobemsg))(display (perform-part strobemsg))(display (content-part strobemsg)))

;;;;;;;;;;;;;;;; NEW EVAL FUNCTION WITH STROBEMSG ;;;;;;;;;;;;;;;;

(define old-eval eval)

(define (eval exp)
  (display "eval: ")(display exp)(newline)
  (if (strobemsg? exp)
      (evaluate-strobemsg exp)
      (old-eval exp)))

;;;;;;;;;;;;;;;; UTILS ;;;;;;;;;;;;;;;;

;(FIRST,SECOND,THIRD,FOURTH,FIFTH LIST)-> ELEMENT, return the first... element of list 
(define (first list) 
  (cond ((not (list? list)) (error "first function needed a list not: " list))
        ((null? list) (error "first function need a list not null " list))
        (else (car list))))
(define (second list) 
  (cond ((not (list? list)) (error "second function needed a list not: " list))
        ((null? list) (error "second function need a list not null " list))
        ((< (length list) 2) (error "second function may be apply on a list of min length 2 " list))
        (else (cadr list))))
(define (third list) 
  (cond ((not (list? list)) (error "third function needed a list not: " list))
        ((null? list) (error "third function need a list not null " list))
        ((< (length list) 3) (error "third function may be apply on a list of min length 3 " list))
        (else (caddr list))))
(define (fourth list) 
  (cond ((not (list? list)) (error "fourth function needed a list not: " list))
        ((null? list) (error "fourth function need a list not null " list))
        ((< (length list) 4) (error "fourth function may be apply on a list of min length 4 " list))
        (else (cadddr list))))
(define (fifth list) 
  (cond ((not (list? list)) (error "fifth function needed a list not: " list))
        ((null? list) (error "fifth function need a list not null " list))
        ((< (length list) 5) (error "fifth function may be apply on a list of min length 5 " list))
        (else (car (cddddr list)))))
(define (sixth list) 
  (cond ((not (list? list)) (error "sixth function needed a list not: " list))
        ((null? list) (error "sixth function need a list not null " list))
        ((< (length list) 6) (error "sixth function may be apply on a list of min length 6 " list))
        (else (car (cdr (cddddr list))))))

;;;;;;;;;;;;;;;; TESTS ;;;;;;;;;;;;;;;;

(define (cube x) (* x x x))
(display "strobemsg.scm loaded")
(newline)

;(eval (make-strobemsg 'me 'you 'order '(+ 2 3))) 
;(eval '(strobemsg me you order (+ 2 3)))
